﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Drawing;
using System.Windows.Forms;
using Infragistics.UltraChart.Core;
using Infragistics.UltraChart.Core.Primitives;
using Infragistics.UltraChart.Resources.Appearance;

namespace ChartCustomText
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            this.ultraChart1.Data.DataSource = GetData();

        }

        private DataTable GetData()
        {
            DataTable dt = new DataTable();

            dt.Columns.Add("Series Label", typeof(string));
            dt.Columns.Add("Actual", typeof(int));
            dt.Columns.Add("Planned", typeof(int));

            DataRow dr = null;
            Random rand = new Random();
            for (int i = 1; i < 6; i++)
            {
                dr = dt.NewRow();
                dr[0] = "task " + i.ToString();
                dr[1] = rand.Next(1, 50);
                dr[2] = rand.Next(1, 50);
                dt.Rows.Add(dr);
            }

            return dt;
        }

        private void ultraChart1_FillSceneGraph(object sender, Infragistics.UltraChart.Shared.Events.FillSceneGraphEventArgs e)
        {
            SceneGraph sceneGraph = e.SceneGraph;
            Box actual;
            Box planned;
            Infragistics.UltraChart.Core.Primitives.Text textPrimitive;
            NumericDataPoint actualDataPoint;
            NumericDataPoint plannedDataPoint;
            List<Infragistics.UltraChart.Core.Primitives.Text> primitivesToAdd = new List<Text>(5);
            double difference;

            foreach (Primitive p in sceneGraph)
            {
                actual = p as Box;
                if (actual != null)
                {
                    actualDataPoint = actual.DataPoint as NumericDataPoint;
                    
                    if (actualDataPoint != null && actualDataPoint.Label == "Actual")
                    {
                        int index = sceneGraph.IndexOf(actual);
                        if (index != -1 && index + 1 < sceneGraph.Count)
                        {
                            planned = sceneGraph[index + 1] as Box;
                            if (planned != null)
                            {
                                plannedDataPoint = planned.DataPoint as NumericDataPoint;
                                if (plannedDataPoint != null && plannedDataPoint.Label == "Planned")
                                {
                                    difference = actualDataPoint.Value - plannedDataPoint.Value;
                                    Point location = GetLocation(actual, planned);
                                    textPrimitive = new Text(location, difference.ToString());
                                    if (difference > 0.0)
                                    {
                                        // bad planning
                                        textPrimitive.PE.Fill = Color.Red;
                                    }
                                    primitivesToAdd.Add(textPrimitive);
                                }
                            }
                        }
                    }
                }
            }

            foreach (Text text in primitivesToAdd)
            {
                sceneGraph.Add(text);
            }
        }

        private Point GetLocation(Box actual, Box planned)
        {
            Point point = new Point();
            
            int top = Math.Min(actual.rect.Top, planned.rect.Top);
            point.Y = top - 5;

            int left = (actual.rect.Left + planned.rect.Right) / 2; 
            // should probably also account for the text length and set the midpoint of that to this point...
            point.X = left;

            return point;
        }
    }
}
